"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault").default;
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createBlockCategoriesCrud = void 0;
var _handlerGraphql = require("@webiny/handler-graphql");
var _error = _interopRequireDefault(require("@webiny/error"));
var _pubsub = require("@webiny/pubsub");
var _validation = require("./blockCategories/validation");
var _utils = require("@webiny/utils");
const createBlockCategoriesCrud = params => {
  const {
    context,
    storageOperations,
    blockCategoriesPermissions,
    getLocaleCode,
    getTenantId
  } = params;
  const onBeforeBlockCategoryCreate = (0, _pubsub.createTopic)();
  const onAfterBlockCategoryCreate = (0, _pubsub.createTopic)();
  const onBeforeBlockCategoryUpdate = (0, _pubsub.createTopic)();
  const onAfterBlockCategoryUpdate = (0, _pubsub.createTopic)();
  const onBeforeBlockCategoryDelete = (0, _pubsub.createTopic)();
  const onAfterBlockCategoryDelete = (0, _pubsub.createTopic)();
  return {
    /**
     * Lifecycle events
     */
    onBeforeBlockCategoryCreate,
    onAfterBlockCategoryCreate,
    onBeforeBlockCategoryUpdate,
    onAfterBlockCategoryUpdate,
    onBeforeBlockCategoryDelete,
    onAfterBlockCategoryDelete,
    /**
     * This method should return category or null. No error throwing on not found.
     */
    async getBlockCategory(slug, options = {
      auth: true
    }) {
      if (!slug) {
        throw new _error.default({
          message: "Block category slug cannot be empty!",
          code: "GET_BLOCK_CATEGORY_EMPTY_SLUG"
        });
      }
      const {
        auth
      } = options;
      const params = {
        where: {
          slug,
          tenant: getTenantId(),
          locale: getLocaleCode()
        }
      };
      if (!auth) {
        return await storageOperations.blockCategories.get(params);
      }
      await blockCategoriesPermissions.ensure({
        rwd: "r"
      });
      let blockCategory = null;
      try {
        blockCategory = await storageOperations.blockCategories.get(params);
      } catch (ex) {
        throw new _error.default(ex.message || "Could not load block category by slug.", ex.code || "GET_BLOCK_CATEGORY_ERROR", {
          ...(ex.data || {}),
          params
        });
      }
      if (!blockCategory) {
        return null;
      }
      await blockCategoriesPermissions.ensure({
        owns: blockCategory.createdBy
      });
      return blockCategory;
    },
    async listBlockCategories() {
      await blockCategoriesPermissions.ensure({
        rwd: "r"
      });
      const params = {
        where: {
          tenant: getTenantId(),
          locale: getLocaleCode()
        },
        sort: ["createdOn_ASC"]
      };

      // If user can only manage own records, add the createdBy to where values.
      if (await blockCategoriesPermissions.canAccessOnlyOwnRecords()) {
        const identity = context.security.getIdentity();
        params.where.createdBy = identity.id;
      }
      try {
        const [items] = await storageOperations.blockCategories.list(params);
        return items;
      } catch (ex) {
        throw new _error.default(ex.message || "Could not list block categories by given params.", ex.code || "LIST_BLOCK_CATEGORY_ERROR", {
          ...(ex.data || {}),
          params
        });
      }
    },
    async createBlockCategory(input) {
      await blockCategoriesPermissions.ensure({
        rwd: "w"
      });
      const validationResult = await (0, _validation.createBlockCategoryCreateValidation)().safeParseAsync(input);
      if (!validationResult.success) {
        throw (0, _utils.createZodError)(validationResult.error);
      }
      const existingBlockCategory = await this.getBlockCategory(input.slug, {
        auth: false
      });
      if (existingBlockCategory) {
        throw new _handlerGraphql.NotFoundError(`Category with slug "${input.slug}" already exists.`);
      }
      const identity = context.security.getIdentity();
      const data = (0, _utils.removeUndefinedValues)(validationResult.data);
      const blockCategory = {
        ...data,
        createdOn: new Date().toISOString(),
        createdBy: {
          id: identity.id,
          type: identity.type,
          displayName: identity.displayName
        },
        tenant: getTenantId(),
        locale: getLocaleCode()
      };
      try {
        await onBeforeBlockCategoryCreate.publish({
          blockCategory
        });
        const result = await storageOperations.blockCategories.create({
          input: data,
          blockCategory
        });
        await onAfterBlockCategoryCreate.publish({
          blockCategory: result
        });
        return result;
      } catch (ex) {
        throw new _error.default(ex.message || "Could not create block category.", ex.code || "CREATE_BLOCK_CATEGORY_ERROR", {
          ...(ex.data || {}),
          blockCategory
        });
      }
    },
    async updateBlockCategory(slug, input) {
      await blockCategoriesPermissions.ensure({
        rwd: "w"
      });
      const original = await this.getBlockCategory(slug);
      if (!original) {
        throw new _handlerGraphql.NotFoundError(`Block Category "${slug}" not found.`);
      }
      await blockCategoriesPermissions.ensure({
        owns: original.createdBy
      });
      const validationResult = await (0, _validation.createBlockCategoryUpdateValidation)().safeParseAsync(input);
      if (!validationResult.success) {
        throw (0, _utils.createZodError)(validationResult.error);
      }
      const data = (0, _utils.removeUndefinedValues)(validationResult.data);
      const blockCategory = {
        ...original,
        ...data
      };
      try {
        await onBeforeBlockCategoryUpdate.publish({
          original,
          blockCategory
        });
        const result = await storageOperations.blockCategories.update({
          input: data,
          original,
          blockCategory
        });
        await onAfterBlockCategoryUpdate.publish({
          original,
          blockCategory
        });
        return result;
      } catch (ex) {
        throw new _error.default(ex.message || "Could not update block category.", ex.code || "UPDATE_BLOCK_CATEGORY_ERROR", {
          ...(ex.data || {}),
          original,
          blockCategory
        });
      }
    },
    async deleteBlockCategory(slug) {
      await blockCategoriesPermissions.ensure({
        rwd: "d"
      });
      const blockCategory = await this.getBlockCategory(slug);
      if (!blockCategory) {
        throw new _handlerGraphql.NotFoundError(`Block Category "${slug}" not found.`);
      }
      await blockCategoriesPermissions.ensure({
        owns: blockCategory.createdBy
      });

      // Before deleting, we need to check if there are any page blocks in this block category.
      // If so, prevent delete operation.
      const pageBlocks = await this.listPageBlocks({
        where: {
          blockCategory: slug
        }
      });
      if (pageBlocks?.length > 0) {
        throw new _error.default("Cannot delete block category because some page blocks are linked to it.", "BLOCK_CATEGORY_HAS_LINKED_BLOCKS");
      }
      try {
        await onBeforeBlockCategoryDelete.publish({
          blockCategory
        });
        const result = await storageOperations.blockCategories.delete({
          blockCategory
        });
        await onAfterBlockCategoryDelete.publish({
          blockCategory: result
        });
        return result;
      } catch (ex) {
        throw new _error.default(ex.message || "Could not delete block category.", ex.code || "DELETE_BLOCK_CATEGORY_ERROR", {
          ...(ex.data || {}),
          blockCategory
        });
      }
    }
  };
};
exports.createBlockCategoriesCrud = createBlockCategoriesCrud;

//# sourceMappingURL=blockCategories.crud.js.map